"""
MeQuery
=======
"""

from deepcrawl_graphql.accounts.fields import AccountFields
from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.query import Query

from .fields import MeFields


class MeQuery(MeFields, Query):
    """MeQuery class

    Creates a me query instance. "Me" being the authenticated user.
    The instance will be passed to the run_query method in order to execute the query.

    >>> from deepcrawl_graphql.me.me import MeQuery

    >>> me_query = MeQuery(conn)
    >>> me_query.select_me()
    >>> me_query.select_accounts()
    >>> me = conn.run_query(me_query)

    :param conn: Connection.
    :type conn: DeepCrawlConnection
    """

    def __init__(self, conn: DeepCrawlConnection) -> None:
        super().__init__(conn)
        self.query = self.query.me

    def select_me(self):
        """Selects user fields."""
        self.query.select(*self.fields(self.ds))
        return self

    def select_accounts(self, account_query=None):
        """Selects users accounts."""
        account_connection_nodes = self.ds.AccountConnection.nodes.select(*AccountFields.fields(self.ds))
        self.query.select(self.ds.User.accounts.select(account_connection_nodes))
        return self



"""
CrawlQuery
==========
"""

from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.query import Query

from .fields import CrawlFields


class CrawlQuery(CrawlFields, Query):
    """CrawlQuery class

    Creates a crawl query instance.
    The instance will be passed to the run_query method in order to execute the query.

    >>> from deepcrawl_graphql.crawls.crawl import CrawlQuery

    >>> crawl_query = CrawlQuery(conn, "crawl_id")
    >>> crawl_query.select_crawl()
    >>> crawl_query.select_parquet_files("datasource_name")
    >>> crawl_query.select_compared_to()
    >>> crawl = conn.run_query(crawl_query)

    :param conn: Connection.
    :type conn: DeepCrawlConnection
    :param crawl_id: crawl id.
    :type crawl_id: int or str
    """

    def __init__(self, conn: DeepCrawlConnection, crawl_id) -> None:
        super().__init__(conn)
        self.query = self.query.getCrawl.args(id=crawl_id)

    """
    Crawl
    """

    def select_crawl(self):
        """Selects crawl fields."""
        self.query.select(*self.fields(self.ds))
        return self

    def select_parquet_files(self, datasource_name):
        """Selects crawl parquetFiles.

        :param datasource_name: Datasource name.
        :type datasource_name: str
        """
        self.query.select(
            self.ds.Crawl.parquetFiles.args(datasourceName=datasource_name).select(*self.parquet_files_fields(self.ds))
        )
        return self

    def select_crawl_type_counts(self, crawl_types, segment_id=None):
        """Selects crawl fields.

        Not implemented yet.

        :param crawl_types: Crawl type.
        :type crawl_types: str
        :param segment_id: Segment id.
        :type segment_id: int or str
        """
        # Having issues with sending the crawlTypes input.
        raise NotImplementedError()
        args = {"input": {"crawlTypes": crawl_types}}
        if segment_id:
            args["input"]["segmentId"] = segment_id
        self.query.select(self.ds.Crawl.crawlTypeCounts.args(**args).select(*self.crawl_type_counts_fields(self.ds)))
        return self

    def select_crawl_settings(self):
        """Selects crawl crawlSetting."""
        # common fields with ProjectQuery
        raise NotImplementedError()

    def select_compared_to(self):
        """Selects crawl comparedTo."""
        self.query.select(self.ds.Crawl.comparedTo.select(*self.fields(self.ds)))
        return self



"""
Query
=====
"""

from deepcrawl_graphql.api import DeepCrawlConnection


class Query:
    """Query class"""

    def __init__(self, conn: DeepCrawlConnection) -> None:
        self.ds = conn.ds
        self.query = self.ds.Query



"""
ProjectQuery
============
"""

from deepcrawl_graphql.accounts.fields import AccountFields
from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.crawls.fields import CrawlFields
from deepcrawl_graphql.query import Query

from .fields import ProjectFields


class ProjectQuery(ProjectFields, Query):
    """ProjectQuery class

    Creates a project query instance.
    The instance will be passed to the run_query method in order to execute the query.

    >>> from deepcrawl_graphql.projects.project import ProjectQuery

    >>> project_query = ProjectQuery(conn, "project_id")
    >>> project_query.select_project()
    >>> project_query.select_sitemaps()
    >>> project_query.select_advanced_crawl_rate()
    >>> project_query.select_majestic_configuration()
    >>> project_query.select_location()
    >>> project_query.select_google_search_configuration()
    >>> project_query.select_custom_extraction_settings()
    >>> project_query.select_account()
    >>> project_query.select_crawls()
    >>> project = conn.run_query(project_query)

    :param conn: Connection.
    :type conn: DeepCrawlConnection
    :param project_id: project id.
    :type project_id: int or str
    """

    def __init__(self, conn: DeepCrawlConnection, project_id) -> None:
        super().__init__(conn)
        self.query = self.query.getProject.args(id=project_id)

    """
    Project
    """

    def select_project(self):
        """Selects project fields."""
        self.query.select(*self.fields(self.ds))
        return self

    def select_sitemaps(self):
        """Selects project sitemaps."""
        self.query.select(self.ds.Project.sitemaps.select(*self.sitemaps_fields(self.ds)))
        return self

    def select_advanced_crawl_rate(self):
        """Selects project maximumCrawlRateAdvanced."""
        self.query.select(self.ds.Project.maximumCrawlRateAdvanced.select(*self.advanced_crawl_rate_fields(self.ds)))
        return self

    def select_majestic_configuration(self):
        """Selects project majesticConfiguration."""
        self.query.select(self.ds.Project.majesticConfiguration.select(*self.majestic_configuration_fields(self.ds)))
        return self

    def select_location(self):
        """Selects project location."""
        self.query.select(self.ds.Project.location.select(*self.location_fields(self.ds)))
        return self

    def select_last_finished_crawl(self):
        """Selects project lastFinishedCrawl.

        Not implemented yet.
        """
        raise NotImplementedError()

    def select_google_search_configuration(self):
        """Selects project googleSearchConsoleConfiguration."""
        self.query.select(
            self.ds.Project.googleSearchConsoleConfiguration.select(*self.google_search_configuration_fields(self.ds))
        )
        return self

    def select_google_analytics_project_view(self):
        """Selects project googleAnalyticsProjectView.

        Not implemented yet.
        """
        raise NotImplementedError()

    def select_custom_extraction_settings(self):
        """Selects project customExtractions."""
        self.query.select(self.ds.Project.customExtractions.select(*self.custom_extraction_setting_fields(self.ds)))
        return self

    """
    Account
    """

    def select_account(self):
        """Selects project account."""
        self.query.select(self.ds.Project.account.select(*AccountFields.fields(self.ds)))
        return self

    """
    Crawl
    """

    def select_crawls(self):
        """Selects project crawls."""
        crawl_nodes = self.ds.CrawlConnection.nodes.select(*CrawlFields.fields(self.ds))
        self.query.select(self.ds.Project.crawls.select(crawl_nodes))
        return self



class AccountFields:
    @staticmethod
    def fields(ds):
        return (
            ds.Account.id,
            ds.Account.active,
            ds.Account.addressCity,
            ds.Account.addressZip,
            ds.Account.availableCredits,
            ds.Account.country,
            ds.Account.createdAt,
            ds.Account.customHeaderColor,
            ds.Account.customLogo,
            ds.Account.customMenuColor,
            ds.Account.limitLevelsMax,
            ds.Account.limitPagesMax,
            ds.Account.maxCrawlRate,
            ds.Account.name,
            ds.Account.packagePlan,
            ds.Account.phone,
            ds.Account.rawID,
            ds.Account.serpentCrawlerEnabled,
            ds.Account.timezone,
            ds.Account.updatedAt,
            ds.Account.apiCallbackUrl,
            ds.Account.accountManagers,
        )

    @staticmethod
    def settings_fields(ds):
        return (
            ds.AccountSetting.code,
            ds.AccountSetting.dataType,
            ds.AccountSetting.description,
            ds.AccountSetting.limit,
            ds.AccountSetting.name,
            ds.AccountSetting.source,
            ds.AccountSetting.type,
            ds.AccountSetting.unit,
            ds.AccountSetting.visible,
        )

    @staticmethod
    def locations_fields(ds):
        return (
            ds.Location.code,
            ds.Location.enabled,
            ds.Location.id,
            ds.Location.name,
            ds.Location.rawID,
            ds.Location.type,
        )

    @staticmethod
    def subscription_fields(ds):
        return (
            ds.AccountSubscription.analyzeAvailable,
            ds.AccountSubscription.automateAvailable,
            ds.AccountSubscription.automatorAvailable,
            ds.AccountSubscription.billingAt,
            ds.AccountSubscription.googleDataStudioAvailable,
            ds.AccountSubscription.impactAvailable,
            ds.AccountSubscription.jsRenderingAvailable,
            ds.AccountSubscription.monitorAvailable,
            ds.AccountSubscription.segmentationAvailable,
            ds.AccountSubscription.status,
            ds.AccountSubscription.currentBillingPeriod.select(ds.DateTimeRange.start, ds.DateTimeRange.end),
            ds.AccountSubscription.plan.select(
                ds.AccountSubscriptionPlan.code,
                ds.AccountSubscriptionPlan.minCommitmentPeriod,
                ds.AccountSubscriptionPlan.name,
                ds.AccountSubscriptionPlan.period,
                ds.AccountSubscriptionPlan.status,
            ),
        )

    @staticmethod
    def subscription_addons_fields(ds):
        return (
            ds.AccountSubscriptionAddon.code,
            ds.AccountSubscriptionAddon.integrationType,
            ds.AccountSubscriptionAddon.name,
            ds.AccountSubscriptionAddon.type,
            ds.AccountSubscriptionAddon.status,
        )



class MeFields:
    @staticmethod
    def fields(ds):
        return (
            ds.User.id,
            ds.User.username,
            ds.User.email,
            ds.User.firstName,
            ds.User.lastName,
            ds.User.createdAt,
            ds.User.updatedAt,
            ds.User.jobTitle,
            ds.User.overallLimitLevelsMax,
            ds.User.overallLimitPagesMax,
            ds.User.ssoClientId,
            ds.User.termsAgreed,
            ds.User.rawID,
            ds.User.permissions.select(ds.Permission.code),
        )



"""
ReportQuery
===========
"""

from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.query import Query

from .fields import ReportFields, ReportTypeFields


class ReportQuery(ReportFields, Query):
    """ReportQuery class

    Creates a report query instance.
    The instance will be passed to the run_query method in order to execute the query.

    >>> from deepcrawl_graphql.reports.report import ReportQuery

    >>> report_query = ReportQuery(conn, "crawl_id", "report_tamplate_code", "report_type_code")
    >>> report_query.select_report()
    >>> report_query.select_datasource()
    >>> report_query.select_type()
    >>> report_query.select_trend()
    >>> report_query.select_segment()
    >>> report_query.select_report_template()
    >>> conn.run_query(report_query)

    :param conn: Connection.
    :type conn: DeepCrawlConnection
    :param crawl_id: crawl id.
    :type crawl_id: int or str
    :param report_tamplate_code: report template code.
    :type report_tamplate_code: str
    :param report_type_code: report type code.
    :type report_type_code: str
    :param segment_id: segment id.
    :type segment_id: int or str
    """

    def __init__(
        self, conn: DeepCrawlConnection, crawl_id, report_tamplate_code, report_type_code, segment_id=None
    ) -> None:
        super().__init__(conn)
        args = {
            "input": {
                "crawlId": crawl_id,
                "reportTemplateCode": report_tamplate_code,
                "reportTypeCode": report_type_code,
            }
        }
        if segment_id:
            args["input"]["segmentId"] = segment_id
        self.query = self.query.getReport.args(**args)

    def select_report(self):
        """Selects report fields."""
        self.query.select(*self.fields(self.ds))
        return self

    def select_datasource(self):
        """Selects report datasources."""
        self.query.select(*self.datasource_fields(self.ds))
        return self

    def select_type(self):
        """Selects report type."""
        self.query.select(self.ds.Report.type.select(*ReportTypeFields.fields(self.ds)))
        return self

    def select_trend(self):
        """Selects report trend."""
        # Failes if datasourceCode and datasourceCodeEnum is included
        self.query.select(self.ds.Report.trend.select(*self.fields(self.ds)))
        return self

    def select_segment(self):
        """Selects report segment."""
        self.query.select(self.ds.Report.segment.select(*self.segment_fields(self.ds)))
        return self

    def select_report_template(self):
        """Selects report reportTemplate."""
        self.query.select(self.ds.Report.reportTemplate.select(*self.report_template_fields(self.ds)))
        return self



"""
Connection
==========
"""

import pathlib

from gql import Client, gql
from gql.dsl import DSLField, DSLMutation, DSLQuery, DSLSchema, dsl_gql
from gql.transport.requests import RequestsHTTPTransport

from deepcrawl_graphql.exceptions import MissingCredentials

ENDPOINT = "https://graph.deepcrawl.com/"
SCHMEA_PATH = pathlib.Path(__file__).parent.resolve() / "deepcrawl_schema.graphql"
with open(SCHMEA_PATH, "r", encoding="utf-8") as f:
    SCHEMA = f.read()


class DeepCrawlConnection:
    """DeepCrawlConnection class

    Creates a connection instance used for sending GraphQL queries to DeepCrawl.

    >>> from deepcrawl_graphql.api import DeepCrawlConnection

    >>> DeepCrawlConnection("user_key_id", "secret")
    DeepCrawlConnection instance

    >>> DeepCrawlConnection(token="token")
    DeepCrawlConnection instance

    :param user_key_id: user key, used together with secret for authentication.
    :type user_key_id: int or str
    :param secret: secret key, used together with user_key_id for authentication.
    :type secret: str
    :param token: authentication token, if used ignores the user_key_id and secret
    :type token: str
    """

    def __init__(self, user_key_id=None, secret=None, token=None) -> None:
        if not token:
            if not user_key_id or not secret:
                raise MissingCredentials("Eighter token or user_key_id and secret pair must be provided")
            token = self._get_token(user_key_id, secret)

        self.headers = {"x-auth-token": token}
        self.transport = RequestsHTTPTransport(url=ENDPOINT, headers=self.headers)
        self.client = Client(transport=self.transport, schema=SCHEMA)
        self.ds = DSLSchema(self.client.schema)

    def __str__(self) -> str:
        return "DeepCrawlConnection instance"

    def __repr__(self) -> str:
        return "DeepCrawlConnection instance"

    def run_query(self, query):
        """Runs a query.

        >>> conn = DeepCrawlConnection("user_key_id", "secret")

        There are 3 possible ways you can run a query

        Using a query string. You can use the DeepCrawl explorer to construct the string https://graph-docs.deepcrawl.com/graphql/explorer

        >>> query = 'query MyQuery {version}'
        >>> conn.run_query(query)
        {'version': '1.91.1-next.0-en'}

        Using the gql package to construct a dynamic query.

        >>> from gql.dsl import DSLQuery
        >>> query = DSLQuery(conn.ds.Query.me.select(conn.ds.User.id, conn.ds.User.username))
        >>> conn.run_query(query)
        {
            'me': {
                'id': 'id',
                'username': 'email@example.com'
            }
        }
        # For more information about constructing queries with dsl
        # see https://gql.readthedocs.io/en/stable/advanced/dsl_module.html

        Import a query from the deepcrawl_graphql package and use it's prebuild queries.

        >>> from deepcrawl_graphql.me.me import MeQuery
        >>> me_query = MeQuery(conn)
        >>> me_query.select_me()
        >>> conn.run_query(me_query)
        {
            'me': {
                'id': 'id',
                'username': 'email@example.com',
                'email': 'email@example.com',
                'firstName': 'FirstName',
                'lastName': 'LastName',
                'createdAt': '2019-10-27T17:11:17.000Z',
                'updatedAt': '2022-01-15T10:10:38.000Z',
                'jobTitle': None,
                'overallLimitLevelsMax': 1000,
                'overallLimitPagesMax': 10000000,
                'ssoClientId': None,
                'termsAgreed': True,
                'rawID': 'id',
                'permissions': []
            }
        }


        :param query: query object
        :type query: str or Query
        """
        from deepcrawl_graphql.query import Query  # pylint: disable=import-outside-toplevel

        if isinstance(query, Query):
            query = dsl_gql(DSLQuery(query.query))
        if isinstance(query, DSLField):
            query = dsl_gql(DSLQuery(query))
        elif isinstance(query, DSLQuery):
            query = dsl_gql(query)
        elif isinstance(query, str):
            query = gql(query)
        return self.client.execute(query)

    @staticmethod
    def _get_token(user_key_id, secret) -> str:
        transport = RequestsHTTPTransport(url=ENDPOINT)
        client = Client(transport=transport, schema=SCHEMA)
        ds = DSLSchema(client.schema)
        query = dsl_gql(
            DSLMutation(
                ds.Mutation.createSessionUsingUserKey.args(
                    input={
                        "userKeyId": user_key_id,
                        "secret": secret,
                    }
                ).select(ds.Session.token)
            )
        )
        response = client.execute(query)
        return response.get("createSessionUsingUserKey", {}).get("token")



"""
AccountQuery
============
"""

from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.projects.fields import ProjectFields
from deepcrawl_graphql.query import Query

from .fields import AccountFields


class AccountQuery(AccountFields, Query):
    """AccountQuery class

    Creates an accout query instance.
    The instance will be passed to the run_query method in order to execute the query.

    >>> from deepcrawl_graphql.accounts.account import AccountQuery

    >>> account_query = AccountQuery(conn, "id")
    >>> account_query.select_account()
    >>> account_query.select_settings()
    >>> account_query.select_callback_headers()
    >>> account_query.select_feature_flags()
    >>> account_query.select_locations()
    >>> account_query.select_package()
    >>> account_query.select_subscription()
    >>> account_query.select_projects()
    >>> account_query.select_project("project_id")
    >>> account = conn.run_query(account_query)

    :param conn: Connection.
    :type conn: DeepCrawlConnection
    :param account_id: account id.
    :type account_id: int or str
    """

    def __init__(self, conn: DeepCrawlConnection, account_id) -> None:
        super().__init__(conn)
        self.query = self.query.getAccount.args(id=account_id)

    """
    Account
    """

    def select_account(self):
        """Selects account fields."""
        self.query.select(*self.fields(self.ds))
        return self

    def select_settings(self):
        """Selects account accountSettings."""
        self.query.select(self.ds.Account.accountSettings.select(*self.settings_fields(self.ds)))
        return self

    def select_callback_headers(self):
        """Selects account apiCallbackHeaders."""
        self.query.select(
            self.ds.Account.apiCallbackHeaders.select(self.ds.APICallbackHeader.key, self.ds.APICallbackHeader.value)
        )
        return self

    def select_feature_flags(self):
        """Selects account featureFlags."""
        self.query.select(self.ds.Account.featureFlags.select(self.ds.FeatureFlag.name, self.ds.FeatureFlag.enabled))
        return self

    def select_locations(self):
        """Selects account locations."""
        self.query.select(self.ds.Account.locations.select(*self.locations_fields(self.ds)))
        return self

    def select_package(self):
        """Selects account primaryAccountPackage."""
        self.query.select(
            self.ds.Account.primaryAccountPackage.select(
                self.ds.AccountPackage.credits, self.ds.AccountPackage.packageType
            )
        )
        return self

    def select_subscription(self, include_addons=False, integration_type=None):
        """Selects account subscription.

        :param include_addons: If true includes the addons available.
        :type include_addons: bool
        :param integration_type: Selects an addon by integration type
        :type integration_type: str
        """
        self.query.select(self.ds.Account.subscription.select(*self.subscription_fields(self.ds)))
        if include_addons:
            self.query.select(
                self.ds.Account.subscription.select(
                    self.ds.AccountSubscription.addons.select(*self.subscription_addons_fields(self.ds))
                )
            )
        if integration_type:
            self.query.select(
                self.ds.Account.subscription.select(
                    self.ds.AccountSubscription.addonByIntegrationType.args(integrationType=integration_type).select(
                        *self.subscription_addons_fields(self.ds)
                    )
                )
            )
        return self

    """
    Project
    """

    def select_projects(self):
        """Selects account projects."""
        projects_nodes = self.ds.ProjectConnection.nodes.select(*ProjectFields.fields(self.ds))
        self.query.select(self.ds.Account.projects.select(projects_nodes))
        return self

    def select_project(self, project_id):
        """Selects account project by id.

        :param project_id: Project id.
        :type project_id: bool
        """
        self.query.select(self.ds.Account.project.args(id=project_id).select(*ProjectFields.fields(self.ds)))
        return self



"""
Setup.py
"""
from pathlib import Path

import setuptools

this_directory = Path(__file__).parent
install_requires = (this_directory / "requirements.txt").read_text(encoding="utf8")
long_description = (this_directory / "README.md").read_text(encoding="utf8")

setuptools.setup(
    name="deepcrawl_graphql",
    version="0.1.2",
    author="Andrei Mutu",
    author_email="mutuandrei02@gmail.com",
    description="A package to simplify usage of the DeepCrawl GraphQL",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/DeepCrawlSEO/dc_graphql_wrapper",
    packages=setuptools.find_packages(exclude=("tests", "docs")),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=install_requires,
    data_files=["requirements.txt"],
    include_package_data=True,
)



class ReportFields:
    @staticmethod
    def fields(ds):
        return (
            ds.Report.id,
            ds.Report.change,
            ds.Report.changeWeight,
            ds.Report.coreUIUrl,
            ds.Report.crawlId,
            ds.Report.createdAt,
            ds.Report.hasChanges,
            ds.Report.projectId,
            ds.Report.queryVersion,
            ds.Report.rawID,
            ds.Report.rawTrends,
            ds.Report.reportTemplateCode,
            ds.Report.reportTypeCode,
            ds.Report.reportTypeCodeEnum,
            ds.Report.segmentVersion,
            ds.Report.totalRows,
            ds.Report.totalWeight,
            ds.Report.typeCode,
        )

    @staticmethod
    def datasource_fields(ds):
        return (
            ds.Report.datasourceCode,
            ds.Report.datasourceCodeEnum,
        )

    @staticmethod
    def segment_fields(ds):
        return (
            ds.Segment.id,
            ds.Segment.name,
            ds.Segment.crawlUrlFilter,
            ds.Segment.createdAt,
            ds.Segment.filter,
            ds.Segment.lastFailedAt,
            ds.Segment.lastGeneratedAt,
            ds.Segment.rawID,
            ds.Segment.updatedAt,
            ds.Segment.version,
        )

    @staticmethod
    def report_template_fields(ds):
        return (
            ds.ReportTemplate.allowedOutputTypes,
            ds.ReportTemplate.automatorSummary,
            ds.ReportTemplate.beta,
            ds.ReportTemplate.categories,
            ds.ReportTemplate.changeSign,
            ds.ReportTemplate.changeWeight,
            ds.ReportTemplate.code,
            ds.ReportTemplate.createdAt,
            ds.ReportTemplate.datasourceCode,
            ds.ReportTemplate.datasourceCodeEnum,
            ds.ReportTemplate.defaultMetrics,
            ds.ReportTemplate.defaultOrderBy,
            ds.ReportTemplate.deletedAt,
            ds.ReportTemplate.description,
            ds.ReportTemplate.healthscoreCategory,
            ds.ReportTemplate.healthscoreSubcategories,
            ds.ReportTemplate.id,
            ds.ReportTemplate.name,
            ds.ReportTemplate.position,
            ds.ReportTemplate.queryVersion,
            ds.ReportTemplate.rawID,
            ds.ReportTemplate.reportTypes,
            ds.ReportTemplate.summary,
            ds.ReportTemplate.tags,
            ds.ReportTemplate.totalSign,
            ds.ReportTemplate.totalWeight,
            ds.ReportTemplate.updatedAt,
            ds.ReportTemplate.reportCategories.select(
                ds.ReportCategory.code,
                ds.ReportCategory.healthScore,
                ds.ReportCategory.name,
                ds.ReportCategory.parentCode,
                ds.ReportCategory.position,
            ),
            # TODO Add metricsGroupings somewhere
        )


class ReportTypeFields:
    @staticmethod
    def fields(ds):
        return (
            ds.ReportType.code,
            ds.ReportType.id,
            ds.ReportType.name,
            ds.ReportType.position,
            ds.ReportType.rawID,
        )



class CrawlFields:
    @staticmethod
    def fields(ds):
        return (
            ds.Crawl.id,
            ds.Crawl.archivedAt,
            ds.Crawl.archivingAt,
            ds.Crawl.brokenAt,
            ds.Crawl.bucketSlot,
            ds.Crawl.canceledAt,
            ds.Crawl.cancelingAt,
            ds.Crawl.comparedToCrawlId,
            ds.Crawl.coreUIUrl,
            ds.Crawl.crawlLevels,
            ds.Crawl.crawlTypes,
            ds.Crawl.crawlability,
            ds.Crawl.crawlabilityTrend,
            ds.Crawl.crawledAt,
            ds.Crawl.crawledPerSecond,
            ds.Crawl.crawledPhase1At,
            ds.Crawl.crawledPhase2At,
            ds.Crawl.crawlerType,
            ds.Crawl.crawlingAt,
            ds.Crawl.crawlingPhase1At,
            ds.Crawl.crawlingPhase2At,
            ds.Crawl.createdAt,
            ds.Crawl.discoveredUrlsAt,
            ds.Crawl.discoveringUrlsAt,
            ds.Crawl.errorMessage,
            ds.Crawl.erroredAt,
            ds.Crawl.finishedAt,
            ds.Crawl.fsmExtTransitionedAt,
            ds.Crawl.fsmTransitionedAt,
            ds.Crawl.incomplete,
            ds.Crawl.isContainer,
            ds.Crawl.isScheduled,
            ds.Crawl.languageSettings,
            ds.Crawl.languageSettingsAdded,
            ds.Crawl.languageSettingsRemoved,
            ds.Crawl.limitLevelsMax,
            ds.Crawl.limitPagesMax,
            ds.Crawl.maxProcessLinks,
            ds.Crawl.metricsServerId,
            ds.Crawl.migratedAt,
            ds.Crawl.migratingAt,
            ds.Crawl.optimusTransformedAt,
            ds.Crawl.optimusTransformingAt,
            ds.Crawl.pageGroupings,
            ds.Crawl.parquetFilesUrls,
            ds.Crawl.partition,
            ds.Crawl.pauseReason,
            ds.Crawl.pausedReminderSent,
            ds.Crawl.populatedAt,
            ds.Crawl.populatingAt,
            ds.Crawl.postPopulatedAt,
            ds.Crawl.postPopulatingAt,
            ds.Crawl.postTransformedAt,
            ds.Crawl.postTransformingAt,
            ds.Crawl.processInstanceType,
            ds.Crawl.processedAt,
            ds.Crawl.processingAt,
            ds.Crawl.progressCrawled,
            ds.Crawl.progressDiscovered,
            ds.Crawl.progressEnqueued,
            ds.Crawl.progressStepped,
            ds.Crawl.progressUpdatedAt,
            ds.Crawl.projectId,
            ds.Crawl.queueingAt,
            ds.Crawl.queuedAt,
            ds.Crawl.rawID,
            ds.Crawl.readyAt,
            ds.Crawl.refineryEnabled,
            ds.Crawl.refineryProcessedAt,
            ds.Crawl.refineryProcessingAt,
            ds.Crawl.refineryProcessingEnabled,
            ds.Crawl.robotsOverwrite,
            ds.Crawl.scheduleRunningEmailSentAt,
            ds.Crawl.scheduled,
            ds.Crawl.siteExplorer,
            ds.Crawl.startedByApi,
            ds.Crawl.status,
            ds.Crawl.statusEnum,
            ds.Crawl.totalStepLinks,
            ds.Crawl.totalSteps,
            ds.Crawl.transformedAt,
            ds.Crawl.transformingAt,
            ds.Crawl.useEsReportStats,
            ds.Crawl.updatedAt,
            ds.Crawl.useRewriteRules,
            ds.Crawl.useRobotsOverwrite,
            ds.Crawl.version,
            ds.Crawl.progress.select(ds.LevelProgress.level, ds.LevelProgress.stepsProcessed),
            ds.Crawl.createdByUser.select(
                ds.UserInfo.firstName,
                ds.UserInfo.lastName,
                ds.UserInfo.username,
            ),
            ds.Crawl.advancedCrawlStats.select(
                ds.AdvancedCrawlStats.crawledCrawlRate,
                ds.AdvancedCrawlStats.externalCrawlRate,
                ds.AdvancedCrawlStats.internalCrawlRate,
                ds.AdvancedCrawlStats.uncrawledCrawlRate,
            ),
        )

    @staticmethod
    def parquet_files_fields(ds):
        return (
            ds.CrawlParquetFile.datasourceName,
            ds.CrawlParquetFile.expiresAt,
            ds.CrawlParquetFile.files,
        )

    @staticmethod
    def crawl_type_counts_fields(ds):
        return (
            ds.CrawlTypeCount.count,
            ds.CrawlTypeCount.crawlTypes,
        )



from deepcrawl_graphql.api import DeepCrawlConnection
from deepcrawl_graphql.query import Query

from .fields import ReportTypeFields


class ReportTypeQuery(ReportTypeFields, Query):
    def __init__(self, conn: DeepCrawlConnection) -> None:
        super().__init__(conn)
        self.query = self.query.getReportTypes

    def select_report_type(self):
        self.query.select(*self.fields(self.ds))
        return self



class ProjectFields:
    @staticmethod
    def fields(ds):
        return (
            ds.Project.id,
            ds.Project.alertEmails,
            ds.Project.allPagesTotal,
            ds.Project.apiCallbackUrl,
            ds.Project.autoFinalizeOnCrawlLimits,
            ds.Project.compareToCrawl,
            ds.Project.coreUIUrl,
            ds.Project.crawlDisallowedUrls1stLevel,
            ds.Project.crawlHyperlinksExternal,
            ds.Project.crawlHyperlinksInternal,
            ds.Project.crawlImagesExternal,
            ds.Project.crawlImagesInternal,
            ds.Project.crawlNofollowHyperlinks,
            ds.Project.crawlNonHtml,
            ds.Project.crawlNotIncluded1stLevel,
            ds.Project.crawlOtherRelExternal,
            ds.Project.crawlOtherRelInternal,
            ds.Project.crawlRate,
            ds.Project.crawlRedirectsExternal,
            ds.Project.crawlRedirectsInternal,
            ds.Project.crawlRelAmphtmlInternal,
            ds.Project.crawlRelAmphtmlExternal,
            ds.Project.crawlRelCanonicalsExternal,
            ds.Project.crawlRelCanonicalsInternal,
            ds.Project.crawlRelHreflangsExternal,
            ds.Project.crawlRelHreflangsInternal,
            ds.Project.crawlRelMobileExternal,
            ds.Project.crawlRelMobileInternal,
            ds.Project.crawlRelNextPrevExternal,
            ds.Project.crawlRelNextPrevInternal,
            ds.Project.crawlRobotsTxtNoindex,
            ds.Project.crawlScriptsExternal,
            ds.Project.crawlScriptsInternal,
            ds.Project.crawlSitemapsExternal,
            ds.Project.crawlSitemapsInternal,
            ds.Project.crawlStylesheetsExternal,
            ds.Project.crawlStylesheetsInternal,
            ds.Project.crawlTestSite,
            ds.Project.crawlTypes,
            ds.Project.crawlingAt,
            ds.Project.crawlsTotalCount,
            ds.Project.createdAt,
            ds.Project.customHeaderUserAgent,
            ds.Project.customHeaderUserAgentShort,
            ds.Project.customRequestHeaders,
            ds.Project.discoverSitemapsInRobotsTxt,
            ds.Project.duplicatePrecision,
            ds.Project.emptyPageThreshold,
            ds.Project.excludeUrlPatterns,
            ds.Project.finishedAt,
            ds.Project.forceMonitorApp,
            ds.Project.gaDateRange,
            ds.Project.hasTestSitePassword,
            ds.Project.highLogSummaryRequests,
            ds.Project.ignoreInvalidSSLCertificate,
            ds.Project.includeHttpAndHttps,
            ds.Project.includeSubdomains,
            ds.Project.includeUrlPatterns,
            ds.Project.isTestSuite,
            ds.Project.lastCrawlCrawlingAt,
            ds.Project.lastCrawlStatus,
            ds.Project.legacyTasksTotalCount,
            ds.Project.limitLevelsMax,
            ds.Project.limitPagesMax,
            ds.Project.limitUrlsMax,
            ds.Project.logSummaryRequestsHigh,
            ds.Project.logSummaryRequestsLow,
            ds.Project.lowLogSummaryRequests,
            ds.Project.maxBodyContentLength,
            ds.Project.maxContentSize,
            ds.Project.maxDescriptionLength,
            ds.Project.maxExternalLinks,
            ds.Project.maxHtmlSize,
            ds.Project.maxLinks,
            ds.Project.maxLinksProcessed,
            ds.Project.maxLoadTime,
            ds.Project.maxRedirections,
            ds.Project.maxTitleWidth,
            ds.Project.maxUrlLength,
            ds.Project.maximumCrawlRate,
            ds.Project.minContentRatio,
            ds.Project.minDescriptionLength,
            ds.Project.minTitleLength,
            ds.Project.minVisits,
            ds.Project.mobileHomepageUrl,
            ds.Project.mobileUrlPattern,
            ds.Project.name,
            ds.Project.primaryDomain,
            ds.Project.rawID,
            ds.Project.rendererBlockAds,
            ds.Project.rendererBlockAnalytics,
            ds.Project.rendererBlockCustom,
            ds.Project.rendererJsString,
            ds.Project.rendererJsUrls,
            ds.Project.robotsOverwrite,
            ds.Project.schedulesTotalCount,
            ds.Project.secondaryDomains,
            ds.Project.segmentsTotalCount,
            ds.Project.sitePrimary,
            ds.Project.siteTest,
            ds.Project.siteTestUser,
            ds.Project.sitemapUrls,
            ds.Project.startUrls,
            ds.Project.testSiteDomain,
            ds.Project.testSiteUsername,
            ds.Project.testsTotalCount,
            ds.Project.thinPageThreshold,
            ds.Project.totalSteps,
            ds.Project.updatedAt,
            ds.Project.urlRewriteQueryParameters,
            ds.Project.urlRewriteStripFragment,
            ds.Project.urlsExcluded,
            ds.Project.urlsIncluded,
            ds.Project.useMobileSettings,
            ds.Project.useRenderer,
            ds.Project.useRobotsOverwrite,
            ds.Project.useStealthMode,
            ds.Project.useUrlRewriteRules,
            ds.Project.userAgentString,
            ds.Project.userAgentStringMobile,
            ds.Project.userAgentToken,
            ds.Project.userAgentTokenMobile,
            ds.Project.userAgent.select(ds.UserAgent.code, ds.UserAgent.name),
            ds.Project.urlSampling.select(
                ds.UrlSampling.name,
                ds.UrlSampling.regexPattern,
                ds.UrlSampling.samplePercentage,
            ),
            ds.Project.urlRewriteRules.select(
                ds.UrlRewriteRule.caseOption, ds.UrlRewriteRule.matchFrom, ds.UrlRewriteRule.rewriteTo
            ),
            ds.Project.mobileUserAgent.select(ds.UserAgent.code, ds.UserAgent.name),
            ds.Project.customDns.select(ds.CustomDnsSetting.hostname, ds.CustomDnsSetting.ipAddress),
            ds.Project.apiCallbackHeaders.select(ds.APICallbackHeader.key, ds.APICallbackHeader.value),
            ds.Project.alertSetting.select(ds.AlertSetting.code, ds.AlertSetting.name),
        )

    @staticmethod
    def sitemaps_fields(ds):
        return (
            ds.Sitemap.enabled,
            ds.Sitemap.level,
            ds.Sitemap.linksOut,
            ds.Sitemap.parent,
            ds.Sitemap.status,
            ds.Sitemap.type,
            ds.Sitemap.url,
            ds.Sitemap.urlDigest,
        )

    @staticmethod
    def advanced_crawl_rate_fields(ds):
        return (
            ds.AdvancedCrawlRate.type,
            ds.AdvancedCrawlRate.maximumCrawlRate,
            ds.AdvancedCrawlRate.start.select(ds.AdvancedCrawlRateTime.hour),
            ds.AdvancedCrawlRate.end.select(ds.AdvancedCrawlRateTime.day),
        )

    @staticmethod
    def majestic_configuration_fields(ds):
        return (
            ds.MajesticConfiguration.createdAt,
            ds.MajesticConfiguration.updatedAt,
            ds.MajesticConfiguration.enabled,
            ds.MajesticConfiguration.maxRows,
            ds.MajesticConfiguration.useHistoricData,
            ds.MajesticConfiguration.useRootDomain,
        )

    @staticmethod
    def location_fields(ds):
        return (
            ds.Location.id,
            ds.Location.name,
            ds.Location.code,
            ds.Location.enabled,
            ds.Location.rawID,
            ds.Location.type,
        )

    @staticmethod
    def google_search_configuration_fields(ds):
        return (
            ds.GoogleSearchConsoleConfiguration.country,
            ds.GoogleSearchConsoleConfiguration.createdAt,
            ds.GoogleSearchConsoleConfiguration.excludeQueries,
            ds.GoogleSearchConsoleConfiguration.includeQueries,
            ds.GoogleSearchConsoleConfiguration.inheritFromSchedulePeriodOverride,
            ds.GoogleSearchConsoleConfiguration.lastNDays,
            ds.GoogleSearchConsoleConfiguration.minClicks,
            ds.GoogleSearchConsoleConfiguration.searchType,
            ds.GoogleSearchConsoleConfiguration.updatedAt,
            ds.GoogleSearchConsoleConfiguration.useSearchConsolePropertyDomainsAsStartUrls,
        )

    @staticmethod
    def custom_extraction_setting_fields(ds):
        return (
            ds.CustomExtractionSetting.cleanHtmlTags,
            ds.CustomExtractionSetting.label,
            ds.CustomExtractionSetting.matchNumberFrom,
            ds.CustomExtractionSetting.matchNumberTo,
            ds.CustomExtractionSetting.regex,
            ds.CustomExtractionSetting.regexFilter,
            ds.CustomExtractionSetting.reportTemplateCode,
        )
